
/*################################################################################################
 *##################                                                            ##################
 *##################                   Genetic parameters                       ##################
 *##################                                                            ##################
 *##################                     Implementation                         ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*
 * RIGEL: Representation-Independant Genetic Engine Library
 *
 * Copyright (C) 2005-2005 Yann LANDRIN-SCHWEITZER a.k.a. Varkhan
 *
 *
 * This library is free software;  you can redistribute it and/or modify it
 * under  the terms of the  GNU Lesser General Public License  as published
 * by the  Free Software Foundation;  either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This  library  is  distributed  in  the hope that it will be useful, but
 * WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library;  if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Author:   Yann LANDRIN-SCHWEITZER
 * Contact:  varkhan@free.fr
 * Homepage: http://www.varkhan.net/
 */


#include <errno.h>
#include <stdlib.h>

#include <RIGEL/Parameters.h>

#include "Config.h"

#define RIGEL_PARAM_MEMINC 10



//################################################################################
//###
//###  Parameters store memory management
//###



RIGEL_Parameters RIGEL_ParametersAlloc() {
	RIGEL_Parameters param ;
	param = (struct S_RIGEL_Parameters *) malloc(sizeof(struct S_RIGEL_Parameters)) ;
	if(param==NULL) {
		DbgLog("RIGEL_ParametersAlloc","RIGEL_mem",0,NULL) ;
		return NULL ;
	}
	// Default parameters: init to zero
	param->rate_init = 0.0 ;
	param->rate_mutn = 0.0 ;
	param->rate_xovr = 0.0 ;
	param->rate_migr = 0.0 ;
	param->popu_size = 0 ;	// Invalid -- should trigger reject if used
	param->popu_flag = 0 ;
	param->popu_hist = 0 ;
	param->popu_elit = 0 ;
	param->popu_parn = 0 ;
	param->popu_chld = 0 ;
	param->popu_init = 0 ;
	param->popu_fadj = 0.0 ;
	// Named parameters hash
	param->parmem = 0 ;
	param->parnum = 0 ;
	param->parkey = NULL ;
	param->parval = NULL ;
	return param ;
}

RIGEL_Parameters RIGEL_ParametersCopy(RIGEL_Parameters param) {
	RIGEL_Parameters cpypm ;
	cpypm = (struct S_RIGEL_Parameters *) malloc(sizeof(struct S_RIGEL_Parameters)) ;
	if(cpypm==NULL) {
		DbgLog("RIGEL_ParametersCopy","RIGEL_mem",0,NULL) ;
		return NULL ;
	}
	cpypm->rate_init = param->rate_init ;
	cpypm->rate_mutn = param->rate_mutn ;
	cpypm->rate_xovr = param->rate_xovr ;
	cpypm->rate_migr = param->rate_migr ;
	cpypm->popu_size = param->popu_size ;
	cpypm->popu_flag = param->popu_flag ;
	cpypm->popu_hist = param->popu_hist ;
	cpypm->popu_elit = param->popu_elit ;
	cpypm->popu_parn = param->popu_parn ;
	cpypm->popu_chld = param->popu_chld ;
	cpypm->popu_init = param->popu_init ;
	cpypm->popu_fadj = param->popu_fadj ;
	cpypm->parmem = 0 ;
	cpypm->parnum = 0 ;
	cpypm->parkey = NULL ;
	cpypm->parval = NULL ;
	if(param->parmem>0) {
		cpypm->parkey = (char * *) malloc(param->parmem*sizeof(char *)) ;
		if(cpypm->parkey==NULL) {
			free(cpypm) ;
			DbgLog("RIGEL_ParametersCopy","RIGEL_mem",0,NULL) ;
			return NULL ;
		}
		cpypm->parval = (double *) malloc(param->parmem*sizeof(double)) ;
		if(cpypm->parval==NULL) {
			free(cpypm->parkey) ;
			free(cpypm) ;
			DbgLog("RIGEL_ParametersCopy","RIGEL_mem",0,NULL) ;
			return NULL ;
		}
	}
	cpypm->parmem = param->parmem ;
	if(param->parnum>0) {
		unsigned long i = 0 ;
		while(i<param->parnum) {
			unsigned long j = 0 ;
			char * ky ;
			while(param->parkey[i][j]!='\0') j ++ ;
			ky = (char *) malloc((j+1)*sizeof(char)) ;
			if(ky==NULL) {
				while(i>0) {
					i -- ;
					free(cpypm->parkey[i]) ;
				}
				free(cpypm->parkey) ;
				free(cpypm->parval) ;
				free(cpypm) ;
				DbgLog("RIGEL_ParametersCopy","RIGEL_mem",0,NULL) ;
				return NULL ;
			}
			ky[j] = '\0' ;
			while(j>0) {
				j -- ;
				ky[j] = param->parkey[i][j] ;
			}
			cpypm->parkey[i] = ky ;
			cpypm->parval[i] = param->parval[i] ;
			i ++ ;
		}
	}
	cpypm->parnum = param->parnum ;
	return cpypm ;
}

void RIGEL_ParametersFree(RIGEL_Parameters param) {
	if(param!=NULL) {
		unsigned long i ;
		i = param->parnum ;
		if(param->parkey!=NULL) while(i>0) {
			i -- ;
			free(param->parkey[i]) ;
			param->parkey[i] = NULL ;
		}
		free(param->parkey) ;
		param->parkey = NULL ;
		free(param->parval) ;
		param->parmem = 0 ;
		param->parnum = 0 ;
		param->popu_size = 0 ;
		free(param) ;
	}
}





//################################################################################
//###
//###  Access to hash-stored parameters
//###



double RIGEL_ParametersGetHash(RIGEL_Parameters param, char * name) {
	unsigned long i, j ;
	char * ky ;
	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersGetHash","RIGEL_arg",0,"NULL RIGEL_Parameters argument") ;
		return -1 ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersGetHash","RIGEL_arg",0,"NULL name string") ;
		return -1 ;
	}
	i = param->parnum ;
	while(i>0) {
		i -- ;
		j = 0 ;
		ky = param->parkey[i] ;
		while(name[j]!='\0' && ky[j]==name[j]) j ++ ;
		if(name[j]=='\0' && ky[j]=='\0') {
			// Found -> return value
			return param->parval[i] ;
		}
	}
	// Not found -> return invalid
	return -1 ;
}

double RIGEL_ParametersSetHash(RIGEL_Parameters param, char * name, double value) {
	unsigned long i, j ;
	char * ky ;
	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersSetHash","RIGEL_arg",0,"NULL RIGEL_Parameters argument") ;
		return -1 ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersSetHash","RIGEL_arg",0,"NULL name string") ;
		return -1 ;
	}
	if(value<0) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersSetHash","RIGEL_arg",0,"Invalid negative value") ;
		return -1 ;
	}
	i = param->parnum ;
	while(i>0) {
		i -- ;
		j = 0 ;
		ky = param->parkey[i] ;
		while(name[j]!='\0' && ky[j]==name[j]) j ++ ;
		if(name[j]=='\0' && ky[j]=='\0') {
			// Found -> replace
			param->parval[i] = value ;
			return value ;
		}
	}
	// Not found -> create
	if(param->parnum>=param->parmem) {
		char * * nk ;
		double * nv ;
		nk = (char * *) realloc(param->parkey,(param->parmem+RIGEL_PARAM_MEMINC)*sizeof(char *)) ;
		if(nk==NULL) {
			DbgLog("RIGEL_ParametersSetHash","RIGEL_mem",0,NULL) ;
			return -1 ;
		}
		param->parkey = nk ;
		nv = (double *) realloc(param->parval,(param->parmem+RIGEL_PARAM_MEMINC)*sizeof(double)) ;
		if(nv==NULL) {
			DbgLog("RIGEL_ParametersSetHash","RIGEL_mem",0,NULL) ;
			return -1 ;
		}
		param->parval = nv ;
		param->parmem += RIGEL_PARAM_MEMINC ;
	}
	i = 0 ;
	while(name[i]!='\0') i ++ ;
	ky = (char *) malloc((i+1)*sizeof(char)) ;
	if(ky==NULL) {
		DbgLog("RIGEL_ParametersSetHash","RIGEL_mem",0,NULL) ;
		return -1 ;
	}
	ky[i] = '\0' ;
	while(i>0) {
		i -- ;
		ky[i] = name[i] ;
	}
	param->parkey[param->parnum] = ky ;
	param->parval[param->parnum] = value ;
	param->parnum ++ ;
	return value ;
}

double RIGEL_ParametersDelHash(RIGEL_Parameters param, char * name) {
	unsigned long i, j ;
	char * ky ;
	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersDelHash","RIGEL_arg",0,"NULL RIGEL_Parameters argument") ;
		return -1 ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ParametersDelHash","RIGEL_arg",0,"NULL name string") ;
		return -1 ;
	}
	i = param->parnum ;
	while(i>0) {
		i -- ;
		j = 0 ;
		ky = param->parkey[i] ;
		while(name[j]!='\0' && ky[j]==name[j]) j ++ ;
		if(name[j]=='\0' && ky[j]=='\0') {
			// Found -> delete
			double old = param->parval[i] ;
			free(param->parkey[i]) ;
			param->parnum -- ;
			param->parkey[i] = param->parkey[param->parnum] ;
			param->parval[i] = param->parval[param->parnum] ;
			return old ;
		}
	}
	// Not found -> return invalid
	return -1 ;
}





//################################################################################
//###
//###  Initialisation and string-based setting of parameters
//###



RIGEL_Parameters RIGEL_ParametersSetDefaults(RIGEL_Parameters param) {
	if(param==NULL) {
		param = (struct S_RIGEL_Parameters *) malloc(sizeof(struct S_RIGEL_Parameters)) ;
		if(param==NULL) {
			DbgLog("RIGEL_ParametersSetDefaults","RIGEL_mem",0,NULL) ;
			return NULL ;
		}
		// Hash parameters hash
		param->parmem = 0 ;
		param->parnum = 0 ;
		param->parkey = NULL ;
		param->parval = NULL ;
	}
	// Default parameters
	param->rate_init = 1.0 ;
	param->rate_mutn = 1.0 ;
	param->rate_xovr = 1.0 ;
	param->rate_migr = 0.0 ;
	param->popu_size = 3 ;
	param->popu_flag = 0 ;
	param->popu_hist = 0 ;
	param->popu_elit = 1 ;
	param->popu_parn = 1 ;
	param->popu_chld = 2 ;
	param->popu_init = 0 ;
	param->popu_fadj = 0.0 ;
	return param ;
}



#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>


double RIGEL_ParametersGetDouble(RIGEL_Parameters param, char * name) {
	if(param==NULL) {
		errno = EINVAL ;
		return -1 ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		return -1 ;
	}
	if(name[0]=='\0') {
		return 0 ;
	}
	if(!strcmp(name,"rate_init")) {
		return param->rate_init ;
	}
	if(!strcmp(name,"rate_mutn")) {
		return param->rate_mutn ;
	}
	if(!strcmp(name,"rate_xovr")) {
		return param->rate_xovr ;
	}
	if(!strcmp(name,"rate_migr")) {
		return param->rate_migr ;
	}
	if(!strcmp(name,"popu_size")) {
		return param->popu_size ;
	}
	if(!strcmp(name,"popu_hist")) {
		return param->popu_hist ;
	}
	if(!strcmp(name,"popu_elit")) {
		return param->popu_elit ;
	}
	if(!strcmp(name,"popu_parn")) {
		return param->popu_parn ;
	}
	if(!strcmp(name,"popu_chld")) {
		return param->popu_chld ;
	}
	if(!strcmp(name,"popu_init")) {
		return param->popu_init ;
	}
	if(!strcmp(name,"popu_flag")) {
		// Technically, the 'unsigned long' should fit completely inside a 'double'
		// so simply casting back to 'unsigned long' should give the right value
		return param->popu_flag ;
	}
	return RIGEL_ParametersGetHash(param,name) ;
}

char * RIGEL_ParametersGetString(RIGEL_Parameters param, char * name) {
	double v ;
	char * str = NULL ;
	if(param==NULL) {
		errno = EINVAL ;
		return NULL ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		return NULL ;
	}
	if(name[0]=='\0') {
		str = malloc(sizeof(char)) ;
		str[0] = '\0' ;	
		return str ;
	}
	if(!strcmp(name,"rate_init")) {
		str = malloc(17*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,16,"%f",param->rate_init) ;
		return str ;
	}
	if(!strcmp(name,"rate_mutn")) {
		str = malloc(17*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,16,"%f",param->rate_mutn) ;
		return str ;
	}
	if(!strcmp(name,"rate_xovr")) {
		str = malloc(17*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,16,"%f",param->rate_xovr) ;
		return str ;
	}
	if(!strcmp(name,"rate_migr")) {
		str = malloc(17*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,16,"%f",param->rate_migr) ;
		return str ;
	}
	if(!strcmp(name,"popu_size")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_size) ;
		return str ;
	}
	if(!strcmp(name,"popu_hist")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_hist) ;
		return str ;
	}
	if(!strcmp(name,"popu_elit")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_elit) ;
		return str ;
	}
	if(!strcmp(name,"popu_parn")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_parn) ;
		return str ;
	}
	if(!strcmp(name,"popu_chld")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_chld) ;
		return str ;
	}
	if(!strcmp(name,"popu_init")) {
		str = malloc(11*sizeof(char)) ;
		if(str==NULL) return NULL ;
		snprintf(str,10,"%lu",param->popu_init) ;
		return str ;
	}
	if(!strcmp(name,"popu_flag")) {
		str = malloc(129*sizeof(char)) ;
		if(str==NULL) return NULL ;
		str[0] = '\0' ;
		if(param->popu_flag&RIGEL_PARAM_SORTDESC) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SORTDESC",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_SELPROPN) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SELPROPN",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_SELRANKG) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SELRANKG",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_SELTOURN) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SELTOURN",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_REPPROPN) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"REPPROPN",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_REPRANKG) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"REPRANKG",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_REPTOURN) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"REPTOURN",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_SELCOMMA) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SELCOMMA",128-strlen(str)) ;
		}
		else {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"SELPLUS",128-strlen(str)) ;
		}
		if(param->popu_flag&RIGEL_PARAM_REPCOMMA) {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"REPCOMMA",128-strlen(str)) ;
		}
		else {
			if(str[0]!='\0') strncat(str,"|",128-strlen(str)) ;
			strncat(str,"REPPLUS",128-strlen(str)) ;
		}
		return str ;
	}
	v = RIGEL_ParametersGetHash(param,name) ;
	str = malloc(17*sizeof(char)) ;
	if(str==NULL) return NULL ;
	snprintf(str,16,"%f",v) ;
	return str ;
}

unsigned long RIGEL_ParametersSet(RIGEL_Parameters param, char * name, char * val) {
	double o ;
	if(param==NULL) {
		errno = EINVAL ;
		return -1 ;
	}
	if(name==NULL) {
		errno = EINVAL ;
		return -1 ;
	}
	if(val==NULL) {
		errno = EINVAL ;
		return -1 ;
	}
	if(name[0]=='\0') {
		return 0 ;
	}
	if(!strcmp(name,"rate_init")) {
		double v = atof(val) ;
		o = param->rate_init ;
		if(v<0) return 0 ;
		if(v>1) return 0 ;
		param->rate_init = v ;
		return o ;
	}
	if(!strcmp(name,"rate_mutn")) {
		double v = atof(val) ;
		o = param->rate_mutn ;
		if(v<0) return 0 ;
		if(v>1) return 0 ;
		param->rate_mutn = v ;
		return o ;
	}
	if(!strcmp(name,"rate_xovr")) {
		double v = atof(val) ;
		o = param->rate_xovr ;
		if(v<0) return 0 ;
		if(v>1) return 0 ;
		param->rate_xovr = v ;
		return o ;
	}
	if(!strcmp(name,"rate_migr")) {
		double v = atof(val) ;
		o = param->rate_migr ;
		if(v<0) return 0 ;
		if(v>1) return 0 ;
		param->rate_migr = v ;
		return o ;
	}
	if(!strcmp(name,"popu_size")) {
		unsigned long v = atol(val) ;
		o = param->popu_size ;
		if(v<=0) return 0 ;
		param->popu_size = v ;
		return o ;
	}
	if(!strcmp(name,"popu_hist")) {
		unsigned long v = atol(val) ;
		o = param->popu_hist ;
		if(v<0) return 0 ;
		param->popu_hist = v ;
		return o ;
	}
	if(!strcmp(name,"popu_elit")) {
		unsigned long v = atol(val) ;
		o = param->popu_elit ;
		if(v<0) return 0 ;
		param->popu_elit = v ;
		return o ;
	}
	if(!strcmp(name,"popu_parn")) {
		unsigned long v = atol(val) ;
		o = param->popu_parn ;
		if(v<0) return 0 ;
		param->popu_parn = v ;
		return o ;
	}
	if(!strcmp(name,"popu_chld")) {
		unsigned long v = atol(val) ;
		o = param->popu_chld ;
		if(v<0) return 0 ;
		param->popu_chld = v ;
		return o ;
	}
	if(!strcmp(name,"popu_init")) {
		unsigned long v = atol(val) ;
		o = param->popu_init ;
		if(v<0) return 0 ;
		param->popu_init = v ;
		return o ;
	}
	if(!strcmp(name,"popu_flag")) {
		unsigned long i = 0 ;
		char c = '+' ;
		char s[16] ;
		unsigned long j ;
		o = param->popu_flag ;
//		param->popu_flag = 0 ;
		if(!strncmp("0x",val,2)) {
			param->popu_flag = strtoul(val,NULL,16) ;
		}
		else while(val[i]!='\0') {
			// Clear garbage chars at beginning
			while( (val[i]!='\0') && val[i]!='+' && val[i]!='-' &&
			  ( (val[i]<'a' || val[i]>'z') && (val[i]<'A' || val[i]>'Z') )
			     ) { i ++ ; }
			// Test whether this is a 'set' or an 'unset' operation
			// Default is 'set'
			c = '+' ;
			if(val[i]=='+' || val[i]=='-') { c = val[i] ; i ++ ; }
			// Find and copy the word that tells us which flag is to be set/unset
			j = 0 ;
			while( (val[i+j]!='\0') && (j<15) &&
			  ( (val[i+j]>='a' && val[i+j]<='z') || (val[i+j]>='A' && val[i+j]<='Z') )
			  ) {
				s[j] = val[i+j] ;
				j ++ ;
			}
			s[j] = '\0' ;
			// Finish to gobble up the word (if it is a long garbage word)
			while( (val[i+j]!='\0') &&
			  ( (val[i+j]>='a' && val[i+j]<='z') || (val[i+j]>='A' && val[i+j]<='Z') )
			     ) { j ++ ; }
			printf("Flag: %c \"%s\"\n",c,s) ;
			if(s[0]=='\0') {
				//Do nothing
			}
			else if(!strcmp(s,"SORTDESC")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_SORTDESC ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_SORTDESC ; break ;
				}
			}
			else if(!strcmp(s,"SELPROPN")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_SELPROPN ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_SELPROPN ; break ;
				}
			}
			else if(!strcmp(s,"SELRANKG")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_SELRANKG ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_SELRANKG ; break ;
				}
			}
			else if(!strcmp(s,"SELTOURN")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_SELTOURN ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_SELTOURN ; break ;
				}
			}
			else if(!strcmp(s,"REPPROPN")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_REPPROPN ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_REPPROPN ; break ;
				}
			}
			else if(!strcmp(s,"REPRANKG")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_REPRANKG ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_REPRANKG ; break ;
				}
			}
			else if(!strcmp(s,"REPTOURN")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_REPTOURN ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_REPTOURN ; break ;
				}
			}
			else if(!strcmp(s,"SELCOMMA")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_SELCOMMA ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_SELCOMMA ; break ;
				}
			}
			else if(!strcmp(s,"SELPLUS")) {
				switch(c) {
				case '+': param->popu_flag &= ~RIGEL_PARAM_SELCOMMA ; break ;
				case '-': param->popu_flag |= RIGEL_PARAM_SELCOMMA ; break ;
				}
			}
			else if(!strcmp(s,"REPCOMMA")) {
				switch(c) {
				case '+': param->popu_flag |= RIGEL_PARAM_REPCOMMA ; break ;
				case '-': param->popu_flag &= ~RIGEL_PARAM_REPCOMMA ; break ;
				}
			}
			else if(!strcmp(s,"REPPLUS")) {
				switch(c) {
				case '+': param->popu_flag &= ~RIGEL_PARAM_REPCOMMA ; break ;
				case '-': param->popu_flag |= RIGEL_PARAM_REPCOMMA ; break ;
				}
			}
			// Clear up the word
			i += j ;
		}
		return o ;
	}
	// Only set through this interface Hash parameters that already have been defined by SetHash
	o = RIGEL_ParametersGetHash(param,name) ;
	if(o>=0) {
		double v = atof(val) ;
		if(v<0) return 0 ;
		RIGEL_ParametersSetHash(param,name,v) ;
	}
	return o ;
}




//################################################################################
//###
//###  Argument parsing
//###



#include <stdarg.h>

void RIGEL_ParametersParse(RIGEL_Parameters param, ...) {
	char * arg ;
	char * val ;
	va_list ap ;
	va_start(ap,param) ;
	arg = (char *) va_arg(ap, char *) ;
	while(arg!=NULL) {
		val = (char *) va_arg(ap, char *) ;
		RIGEL_ParametersSet(param,arg,val) ;
		arg = (char *) va_arg(ap, char *) ;
	}
	va_end(ap) ;
}

void RIGEL_ParametersParseArgs(RIGEL_Parameters param, int * argc, char * * * argv) {
	unsigned long i = 0 ;
	unsigned long k ;
	while((i+1)<(*argc)) {
		if((*argv)[i][0]=='+' && RIGEL_ParametersSet(param,(*argv)[i]+1,(*argv)[i+1])) {
			k = i+2 ;
			while(k<(*argc)) {
				(*argv)[k-2] = (*argv)[k] ;
				k ++ ;
			}
			(*argc) -= 2 ;
		}
		else i ++ ;
	}
}

unsigned long RIGEL_ReadParameters(RIGEL_Parameters param, int fd) {
	char eof = 0 ;
	char eol = 0 ;
	char * buf = NULL ;
	char * var ;
	char * val ;
	unsigned long mem = 0 ;
	unsigned long len = 0 ;
	unsigned long n = 0 ;
	unsigned long i, j ;
	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_ReadParameters","RIGEL_arg",0,"NULL parameters argument") ;
		return -1 ;
	}
	if(fd==-1) {
		errno = EINVAL ;
		DbgLog("RIGEL_ReadParameters","RIGEL_arg",0,"Invalid file descriptor") ;
		return -1 ;
	}
	while(!eof) {
		i = 0 ;
		while(!eol && !eof) {
			if(len>=mem) {
				buf = realloc(buf,mem+1024) ;
				if(buf==NULL) {
					DbgLog("RIGEL_ReadParameters","RIGEL_mem",0,NULL) ;
					return -1 ;
				}
				mem += 1024 ;
			}
			j = read(fd,buf+len,mem-len) ;
			if(j==-1) {
				free(buf) ;
				DbgLog("RIGEL_ReadParameters","RIGEL_mem",0,NULL) ;
				return -1 ;
			}
			if(j<(mem-len)) eof = 1 ;
			len += j ;
			while(i<len) {
				if(buf[i]=='\n') {
					eol = 1 ;
					break ;
				}
				i ++ ;
			}
		}
		// We have either an eol, or eof, and i is at the end of line / file
		buf[i] = '\0' ; i ++ ;
		j = 0 ;
		while(buf[j]==' ' && buf[j]!='\0') j ++ ;
		var = buf+j ;
		while(buf[j]!=' ' && buf[j]!='=' && buf[j]!='\0') j ++ ;
		if(buf[j]=='\0') continue ;
		buf[j] = '\0' ; j ++ ;
		while((buf[j]==' '||buf[j]=='=') && buf[j]!='\0') j ++ ;
		val = buf+j ;
		while(buf[j]!='\0') j ++ ;
		buf[j] = '\0' ;
		RIGEL_ParametersSet(param,var,val) ;
		n ++ ;
		// Clean up the buffer,moving the unused part at the beginning
		j = 0 ;
		while(i<len) {
			buf[j] = buf[i] ;
			i ++ ;
			j ++ ;
		}
		len = j ;
	}
	free(buf) ;
	return n ;
}

unsigned long RIGEL_WriteParameters(RIGEL_Parameters param, int fd) {
#define CHARNUM 512
	char s[CHARNUM+1] ;
	char * p ;
	unsigned long i, n = 0 ;
	unsigned long t, r ;
	char f ;

#define PRINT_VAL(name,format) \
	t = snprintf(s,CHARNUM, #name " = " format "\n",param->name) ; \
	if(t==-1) { \
		DbgLog("RIGEL_WriteParameters","RIGEL_sys",0,NULL) ; \
		return -1 ; \
	} \
	r = write(fd,s,t) ; \
	if(r==-1) { \
		DbgLog("RIGEL_WriteParameters","RIGEL_sys",0,NULL) ; \
		return -1 ; \
	} \
	n += r ; \
	if(t>=CHARNUM || r<t) { \
		return n ; \
	}

	if(param==NULL) {
		errno = EINVAL ;
		DbgLog("RIGEL_WriteParameters","RIGEL_arg",0,"NULL parameters argument") ;
		return -1 ;
	}
	if(fd==-1) {
		errno = EINVAL ;
		DbgLog("RIGEL_WriteParameters","RIGEL_arg",0,"Invalid file descriptor") ;
		return -1 ;
	}
	PRINT_VAL(popu_size,"%ld") ;
	PRINT_VAL(popu_hist,"%ld") ;
	PRINT_VAL(popu_elit,"%ld") ;
	PRINT_VAL(popu_parn,"%ld") ;
	PRINT_VAL(popu_chld,"%ld") ;
	PRINT_VAL(popu_init,"%ld") ;

	t = 0 ;
	p = "popu_flag =" ;
	i = 0 ;
	while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
	f = ' ' ;
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SORTDESC) {
		s[t] = f ; t ++ ;
		p = "SORTDESC" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELPROPN) {
		s[t] = f ; t ++ ;
		p = "SELPROPN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELRANKG) {
		s[t] = f ; t ++ ;
		p = "SELRANKG" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_SELTOURN) {
		s[t] = f ; t ++ ;
		p = "SELTOURN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPPROPN) {
		s[t] = f ; t ++ ;
		p = "REPPROPN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPRANKG) {
		s[t] = f ; t ++ ;
		p = "REPRANKG" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else if(t<CHARNUM && param->popu_flag&RIGEL_PARAM_REPTOURN) {
		s[t] = f ; t ++ ;
		p = "REPTOURN" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	if(t<CHARNUM) { if(param->popu_flag&RIGEL_PARAM_SELCOMMA) {
		s[t] = f ; t ++ ;
		p = "SELCOMMA" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else {
		s[t] = f ; t ++ ;
		p = "SELPLUS" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	} }
	if(t<CHARNUM) { if(param->popu_flag&RIGEL_PARAM_REPCOMMA) {
		s[t] = f ; t ++ ;
		p = "REPCOMMA" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	}
	else {
		s[t] = f ; t ++ ;
		p = "REPPLUS" ;
		i = 0 ;
		while(p[i]!='\0' && t<CHARNUM) { s[t] = p[i] ; t ++ ; i ++ ; }
		f = '|' ;
	} }
	s[t] = '\n' ; t ++ ;
	r = write(fd,s,t) ;
	if(r==-1) {
		DbgLog("RIGEL_WriteParameters","RIGEL_sys",0,NULL) ;
		return -1 ;
	}
	n += r ;
	if(t>=CHARNUM || r<t) {
		return n ;
	}

	// Printing rates
	PRINT_VAL(rate_init,"%f") ;
	PRINT_VAL(rate_mutn,"%f") ;
	PRINT_VAL(rate_xovr,"%f") ;
	PRINT_VAL(rate_migr,"%f") ;

	// Printing misc
	i = 0 ;
	while(i<param->parnum) {
		t = snprintf(s,CHARNUM,"%s = %f\n",param->parkey[i],param->parval[i]) ;
		if(t==-1) {
			DbgLog("RIGEL_WriteParameters","RIGEL_sys",0,NULL) ;
			return -1 ;
		}
		r = write(fd,s,t) ;
		if(r==-1) {
			DbgLog("RIGEL_WriteParameters","RIGEL_sys",0,NULL) ;
			return -1 ;
		}
		n += r ;
		if(t>=CHARNUM || r<t) {
			return n ;
		}
		i ++ ;
	}
	return n ;
#undef CHARNUM
#undef PRINT_VAL
}


